@@ -45,23 +45,34 @@ use crate::{
     utils::CommandParams,
 };
 use error::Error;
-use std::{fs, path::PathBuf, process::Output, vec};
+use std::{
+    fs::{self, File},
+    io::Write,
+    path::PathBuf,
+    process::Output,
+    vec,
+};
 use structopt::StructOpt;
 use utils::{exec_command, get_current_dir};
 
 /// Get a tuple with config path and the config str
 fn get_config_path_and_str(config_path: Option<PathBuf>) -> Result<(PathBuf, String), Error> {
     let config_path = match config_path {
-        Some(c) => c,
+        Some(c) => Some(c),
         None => get_config_path()?,
     };
 
-    let config_str = match fs::read_to_string(&config_path) {
-        Err(a) => return Err(Error::NoFile(config_path, a)),
-        Ok(s) => s,
-    };
+    match config_path {
+        Some(e) => {
+            let config_str = match fs::read_to_string(&e) {
+                Err(a) => return Err(Error::NoFile(e, a)),
+                Ok(s) => s,
+            };
 
-    Ok((config_path, config_str))
+            Ok((e, config_str))
+        }
+        None => Err(Error::NoConfigFile),
+    }
 }
 
 /// Executor
@@ -71,35 +82,64 @@ pub fn run() -> Result<(), Error> {
     let config_path = opt.config_path;
     let debug = opt.debug;
 
-    let (config_path, config_str) = get_config_path_and_str(config_path)?;
-
-    let pwd = get_current_dir()?;
-
-    let entries = parse_toml(&config_str)?;
-
-    let mut this_dir = vec![];
-    for entry in &entries {
-        // Check if the entry path is IN the pwd
-        // This allows dipse to work when inside a nested system
-        if pwd.starts_with(entry.0) {
-            this_dir.push(entry)
-        }
-    }
-
-    if this_dir.is_empty() {
-        return Err(Error::NoConfigForPath(pwd));
-    }
-
-    this_dir.sort_by(|a, b| a.0.cmp(b.0));
-
-    let (_path, entry) = this_dir[0];
-
     if let Some(sub_cmd) = opt.sub_cmd {
         match sub_cmd {
+            SubOpt::Init => {
+                let config_path = match config_path {
+                    Some(c) => Some(c),
+                    None => get_config_path()?,
+                };
+                match config_path {
+                    Some(c) => return Err(Error::ConfigExist(c)),
+                    None => {
+                        let curr_dir = get_current_dir()?;
+
+                        let new_p = curr_dir.join(".d.toml");
+                        let mut f = match File::create(&new_p) {
+                            Err(e) => return Err(Error::ConfigFileCreation(new_p, e)),
+                            Ok(e) => e,
+                        };
+
+                        if let Err(e) = writeln!(f, "[\"{}\"]", curr_dir.display()) {
+                            return Err(Error::ConfigFileWrite(new_p, e));
+                        }
+
+                        if let Err(e) = f.flush() {
+                            return Err(Error::ConfigFileWrite(new_p, e));
+                        }
+                    }
+                };
+            }
             SubOpt::Edit => {
+                let (config_path, _config_str) = get_config_path_and_str(config_path)?;
                 start_editor(config_path)?;
             }
             SubOpt::Crud(crud) => {
+                let (config_path, config_str) = get_config_path_and_str(config_path)?;
+
+                let pwd = get_current_dir()?;
+
+                let entries = parse_toml(&config_str)?;
+
+                let mut this_dir = vec![];
+                for entry in &entries {
+                    let entry_path = config_path.join(entry.0).canonicalize().expect("asdf");
+
+                    // Check if the entry path is IN the pwd
+                    // This allows dipse to work when inside a nested system
+                    if pwd.starts_with(entry_path) {
+                        this_dir.push(entry)
+                    }
+                }
+
+                if this_dir.is_empty() {
+                    return Err(Error::NoConfigForPath(pwd));
+                }
+
+                this_dir.sort_by(|a, b| b.0.cmp(a.0));
+
+                let (_path, _entry) = this_dir[0];
+
                 // All entries in the file
                 let mut new_entries = entries.clone();
 
@@ -136,7 +176,34 @@ pub fn run() -> Result<(), Error> {
                     return Err(Error::ConfigFileWrite(config_path, e));
                 }
             }
-            SubOpt::Other(cmd) => run_cmd(cmd, entry, debug, opt.no_op)?,
+            SubOpt::Other(cmd) => {
+                let (config_path, config_str) = get_config_path_and_str(config_path)?;
+
+                let pwd = get_current_dir()?;
+
+                let entries = parse_toml(&config_str)?;
+
+                let mut this_dir = vec![];
+                for entry in &entries {
+                    let entry_path = config_path.join(entry.0).canonicalize().expect("asdf");
+
+                    // Check if the entry path is IN the pwd
+                    // This allows dipse to work when inside a nested system
+                    if pwd.starts_with(entry_path) {
+                        this_dir.push(entry)
+                    }
+                }
+
+                if this_dir.is_empty() {
+                    return Err(Error::NoConfigForPath(pwd));
+                }
+
+                this_dir.sort_by(|a, b| b.0.cmp(a.0));
+
+                let (_path, entry) = this_dir[0];
+
+                run_cmd(cmd, entry, debug, opt.no_op)?
+            }
         }
     }
 
